<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2006 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "stemming.php";
include_once "config.php";

if (!$cbir_host || !$cbir_port) {
  $search_interface = array("text_general"  => array("idx" => 1, "active_img" => "$theme/navigator.search.general.a.gif",  "img" => "$theme/navigator.search.general.gif",  link => "search.text.general.php",  "alt" => " General Text "),
                            "text_advanced" => array("idx" => 2, "active_img" => "$theme/navigator.search.advanced.a.gif", "img" => "$theme/navigator.search.advanced.gif", link => "search.text.advanced.php", "alt" => " Advanced Text "));
}
else {
  $search_interface = array("text_general"  => array("idx" => 1, "active_img" => "$theme/navigator.search.general.a.gif",  "img" => "$theme/navigator.search.general.gif",  link => "search.text.general.php",  "alt" => " General Text "),
                            "text_advanced" => array("idx" => 2, "active_img" => "$theme/navigator.search.advanced.a.gif", "img" => "$theme/navigator.search.advanced.gif", link => "search.text.advanced.php", "alt" => " Advanced Text "),
                            "image_layout"  => array("idx" => 3, "active_img" => "$theme/navigator.search.layout.a.gif",   "img" => "$theme/navigator.search.layout.gif",   link => "search.image.layout.php",  "alt" => " Image Layout "),
                            "image_color"   => array("idx" => 4, "active_img" => "$theme/navigator.search.color.a.gif",    "img" => "$theme/navigator.search.color.gif",    link => "search.image.color.php",   "alt" => " Image Color "));
}


function build_itemized_search_string($search_string, $identifier, $arguments) {
  $itemized_search_string = "";

  /* translate tabs and new lines into spaces */
  $token = strtok($search_string," ");
  while ($token) {
    if (strncmp($token, "AND", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "AND ";
    } elseif (strncmp($token, "NOT", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "NOT ";
    } elseif (strncmp($token, "OR", 2) == 0) {
      $itemized_search_string = $itemized_search_string . "OR ";
    } else {
      $itemized_search_string = $itemized_search_string . "<a href=\"search.text.general.php?$arguments".$identifier."=$token\">$token</a> ";
    }
    $token = strtok("+ \n\t");
  }

  return $itemized_search_string;
}

function validate_search_string($search_string) {
  $search_string  = str_replace("\\", "", $search_string);
  $search_string  = str_replace("\"", "&quot;", $search_string);
  $search_string  = str_replace("\'", "&quot;", $search_string);
  $search_string  = str_replace("*",  "", $search_string);
  $search_string  = str_replace("?",  "", $search_string);
  return $search_string;
}

function prepare_keyword($keyword, $enable_stemming) {
  global $forbidden_keywords;
  global $ignored_keywords;

  if (is_numeric($keyword))
    return " ILIKE '".$keyword."'";

  /* check for photo identifiers */
  $dash_position = strrpos($keyword, "-");
  if (is_integer($dash_position)) {
    $identifier = substr($keyword, 0, $dash_position);
    $identifier = ltrim($identifier, "0");
    if (is_numeric($identifier))
      return " ILIKE '".$identifier."'";
  }

  /* check if we have to search for exact maches */
  /* quotation marks must be at the beginning */
  /* and the end of the keyword, elsewhere ignored */
  $quot_1 = substr($keyword, 0, 6);
  $quot_2 = substr($keyword, strlen($keyword)-6, 6);
  if (($quot_1 == "&quot;") &&
      ($quot_2 == "&quot;")) {
    $keyword = str_replace("&quot;", "", $keyword);
    $keyword = str_replace(";", " ", $keyword);
    $keyword = str_replace("#@#", " ", $keyword);

    return " =##= '".$keyword."'";
  }

  /* ignore all signle quotation marks */
  $keyword = str_replace("&quot;", "", $keyword);

  /* prune forbidden keywords */
  foreach($forbidden_keywords as $forbidden_keyword) {
    if (strcasecmp($forbidden_keyword, $keyword) == 0) {
      $ignored_keywords = $ignored_keywords . "<a href=\"search.text.general.php?search_type=$search_type&search_string=$keyword\">$keyword</a> ";
      return;
    }
  }

  /* stem keyword */
  if ($enable_stemming == 't') {
    $stemming_engine = new stemming();
    return $stemming_engine->stemstr($keyword);
  }

  return " ~* '".$keyword."'";
}

function extract_keywords($search_string, $enable_stemming) {
  /* detect quotation marks and replace spaces with the #@# marker so */
  /* that strings between quotation marks are treated as one keyword */
  /* when tokenizing the search string for operators */

  $inside_quotation = FALSE;
  $working_copy = $search_string;
  $search_string = "";
  while ($working_copy) {
    if ($inside_quotation) {
      $pos = strpos($working_copy, "&quot;");
      if ($pos === FALSE) {
        $search_string .= $working_copy;
        break;
      }
      else {
        $search_string .= str_replace(" ", "#@#", substr($working_copy, 0, $pos+5));
        $working_copy = substr($working_copy, $pos+5);
        $inside_quotation = FALSE;
      }
    }
    else {
      $pos = strpos($working_copy, "&quot;");
      if ($pos === FALSE) {
        $search_string .= $working_copy;
        break;
      }
      else {
        $search_string .= substr($working_copy, 0, $pos+5);
        $working_copy = substr($working_copy, $pos+5);
        $inside_quotation = TRUE;
      }
    }
  }

  /* tokenize the search string based on operators and spaces */
  $index = 0;
  $keyword = array();

  $token = strtok($search_string," ");
  while ($token) {
    if (strncmp($token, "AND", 3) == 0) {
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
        $keyword = array_merge($keyword, array($index => array("operator"=>"AND", keyword=>"$token")));
    }
    elseif (strncmp($token, "NOT", 3) == 0) {
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
        $keyword = array_merge($keyword, array($index => array("operator"=>"NOT", keyword=>"$token")));
    }
    elseif (strncmp($token, "OR", 2) == 0) {
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
        $keyword = array_merge($keyword, array($index => array("operator"=>"OR", keyword=>"$token")));
    }
    else {
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
        $keyword = array_merge($keyword, array($index => array("operator"=>"AND", keyword=>"$token")));
    }
    $token = strtok("+ \n\t");
    $index ++;
  }

  if (!$keyword)
    return false;

  return $keyword;
}

function build_sql_search_string_keywords($keywords) {
  $string_a = " photo.identifier in (select k.photo from photo_keywords k where k.keyword";
    
  $sql_search_string = "";

  if (!$keywords)
    return $sql_search_string;

  foreach($keywords as $keyword) {
    if (strncmp($keyword['operator'], "AND", 3) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
    } elseif (strncmp($keyword['operator'], "NOT", 3) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s !%s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
    } elseif (strncmp($keyword['operator'], "OR", 2) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s or %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
    } else {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
    }
  }

  if ($sql_search_string != "")
    $sql_search_string = sprintf("(%s)", $sql_search_string);

  //  print "<pre>$sql_search_string</pre>";

  return "$sql_search_string";
}

function build_sql_search_string($keywords, $sql_field_name) {
  $sql_search_string = "";

  if (!$keywords)
    return $sql_search_string;

  foreach($keywords as $keyword) {
    if (strncmp($keyword[operator], "AND", 3) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword[keyword]);
      else
        $sql_search_string = sprintf("%s and %s %s", $sql_search_string, $sql_field_name, $keyword[keyword]);
    }
    elseif (strncmp($keyword[operator], "NOT", 3) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s !%s", $sql_field_name, $keyword[keyword]);
      else
        $sql_search_string = sprintf("%s and %s !%s", $sql_search_string, $sql_field_name, $keyword[keyword]);
    }
    elseif (strncmp($keyword[operator], "OR", 2) == 0) {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword[keyword]);
      else
        $sql_search_string = sprintf("%s or %s %s", $sql_search_string, $sql_field_name, $keyword[keyword]);
    }
    else {
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword[keyword]);
      else
        $sql_search_string = sprintf("%s and %s %s", $sql_search_string, $sql_field_name, $keyword[keyword]);
    }
  }

  if ($sql_search_string != "")
    $sql_search_string = sprintf("(%s)", $sql_search_string);

  return $sql_search_string;
}


function generate_html_users_combo($identifier, $database, $current_user_id, $style) {
  global $user_type;

  $user_list = pg_query($database, "select identifier, last_name, first_name, member_since from users where users.type > $user_type[client] order by last_name, first_name");

  $html_string .= "<select name=\"$identifier\" $style title=\"Search for photos or folders belonging to a specified user.\">";
  $html_string .= "<option value=\"all\">All Users</option>";
  for ($i=0; $i < pg_num_rows($user_list); $i++) {
    $user = pg_fetch_row($user_list, $i);
    if ($user[0] == $current_user_id) {
      $html_string .= "<option value=\"$user[0]\" selected>$user[1], $user[2]</option>";
    }
    else {
      $html_string .= "<option value=\"$user[0]\">$user[1], $user[2]</option>";
    }
  }
  $html_string .= "</select>";
  return $html_string;
}

?>
