
--  Copyright (C) 2002, 2003, 2004, 2005, 2006 Balint Kis (balint@k-i-s.net)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.10 to v2.11 patch
----------------------------------------------------------

-- add support for original protection
alter table photo add column hide_original  char(1) check (hide_original in ('t', 'f'));

-- create image magick options table and populate it with exisiting sharpening values
create table imagemagick_options (
  identifier              integer not null primary key,

  preview_quality         integer,
  preview_colorspace      varchar(20),
  preview_gamma           float,
  preview_sharpen_radius  float,
  preview_sharpen_sigma   float,
  preview_border_width    integer,
  preview_border_height   integer,
  preview_border_color    varchar(6),

  thumb_quality           integer,
  thumb_colorspace        varchar(20),
  thumb_gamma             float,
  thumb_sharpen_radius    float,
  thumb_sharpen_sigma     float,
  thumb_border_width      integer,
  thumb_border_height     integer,
  thumb_border_color      varchar(6)
);

-- create new reference from the users table
alter table users add column imagemagick_options integer references imagemagick_options(identifier);

-- upload imagemagick table with values from preferences and defaults
insert into imagemagick_options
        (identifier,
         preview_quality, preview_colorspace, preview_gamma,
         preview_sharpen_radius, preview_sharpen_sigma,
         preview_border_color, preview_border_width, preview_border_height,
         thumb_quality, thumb_colorspace, thumb_gamma,
         thumb_sharpen_radius, thumb_sharpen_sigma,
         thumb_border_color, thumb_border_width, thumb_border_height)
  select identifier,
         80 as preview_quality, 'RGB' as preview_colorspace, '1.0' as preview_gamma,
         sharpen_image_radius as preview_sharpen_radius, sharpen_image_sigma as preview_sharpen_sigma,
         '000000' as preview_border_color, 0 as preview_border_width, 0 as preview_border_height,
         80 as thumb_quality, 'RGB' as thumb_colorspace, '1.0' as thumb_gamma,
         sharpen_image_radius as thumb_sharpen_radius, sharpen_image_sigma as thumb_sharpen_sigma,
         '000000' as thumb_border_color, 0 as thumb_border_width, 0 as thumb_border_height
         from preferences;

-- update the newly created imagemagick reference that is the same as for the preferences
update users set imagemagick_options = preferences;
alter table users alter column imagemagick_options set not null;

-- create sequence and set it to the same value as the preferences sequence was
create sequence imagemagick_options_id_sequence;
select setval('imagemagick_options_id_sequence',max(imagemagick_options)) from users;

-- we rename colums first since droping was implemented only with 7.3
alter table preferences rename column sharpen_image_radius to drop_me_sharpen_image_radius;
alter table preferences rename column sharpen_image_sigma  to drop_me_sharpen_image_sigma;
alter table preferences rename column sharpen_thumb_radius to drop_me_sharpen_thumb_radius;
alter table preferences rename column sharpen_thumb_sigma  to drop_me_sharpen_thumb_sigma;

-- drop columns which aren't used any longer
alter table preferences drop column drop_me_sharpen_image_radius;
alter table preferences drop column drop_me_sharpen_image_sigma;
alter table preferences drop column drop_me_sharpen_thumb_radius;
alter table preferences drop column drop_me_sharpen_thumb_sigma;
